global int WINDOW_SIZE;
global int HISTORY;

GG.resolution(1200, 600);

1 => GG.camera().posY;
31 => GG.camera().posZ;
GG.camera().clip(.05, HISTORY * 10);

@(250, 214, 165) => vec3 sunrise;
@(135, 206, 235) => vec3 noon;
@(250, 214, 165) => vec3 sunset;
@(4, 26, 64) => vec3 night;

noon / 255. => GG.scene().backgroundColor;

GG.scene().enableFog();
0.001 => GG.scene().fogDensity;
GG.scene().FOG_EXP => GG.scene().fogType;
@(1, 1, 1) => GG.scene().fogColor;

// accumulate samples from mic
adc => Flip accum => blackhole;
// take the FFT
adc => PoleZero dcblock => FFT fft => blackhole;
// set DC blocker
0.95 => dcblock.blockZero;
// set size of flip
WINDOW_SIZE => accum.size;
// set window type and size
WINDOW_SIZE => Windowing.hann => fft.window;
// set FFT size (will automatically zero pad)
WINDOW_SIZE * 2 => fft.size;

// sample array
float samples[0];
// FFT response
complex response[0];

vec3 vertices[WINDOW_SIZE];

// do audio stuff
fun void doAudio() {
  while (true) {
    // upchuck to process accum
    accum.upchuck();
    // get the last window size samples (waveform)
    samples => accum.output;
    // upchuck to take FFT, get magnitude response
    fft.upchuck();
    // get spectrum (as complex values)
    response => fft.spectrum;
    // jump by samples
    WINDOW_SIZE::samp / 2 => now;
  }
}
spork ~ doAudio();

Sun s --> GG.scene();

Moon m --> GG.scene();

Ocean o --> GG.scene();

Cloud c --> GG.scene();
samples @=> c.samples;

Waves ws --> GG.scene();
vertices @=> ws.vertices;

Wave w --> GG.scene();
vertices @=> w.vertices;

Boat b --> GG.scene();
@(-3.25, 1. / 10., 24.75) => b.pos;
vertices @=> b.vertices;

FlyCam f;
0 => f.init;

Narrative n;

false => int narrative;

fun void end() {
	n.end => now;
	false => narrative;

	dac =< accum;
  dac =< dcblock;

  adc => accum;
  adc => dcblock;
}

Sweep sweep;

false => int flycam;
false => int test;
now => time start;
now => time transition;
int phase;
while (true) {
	if (KB.isKeyDown(KB.KEY_ENTER) && !flycam) {
		true => flycam;
    spork ~ f.selfUpdate();
	} else if (KB.isKeyDown(KB.KEY_BACKSPACE) && flycam) {
		false => flycam;
		f.stop();
		1 => GG.camera().posY;
    30 => GG.camera().posZ;
	} else if (KB.isKeyDown(KB.KEY_SPACE) && !narrative) {
		true => narrative;
		adc =< accum;
		adc =< dcblock;

		dac => accum;
		dac => dcblock;
		spork ~ n.play();
		spork ~ end();
	} else if (KB.isKeyDown(KB.KEY_T) && !test) {
		true => test;
		adc =< accum;
    adc =< dcblock;

    dac => accum;
    dac => dcblock;
    spork ~ sweep.start();
  } else if (KB.isKeyDown(KB.KEY_S) && !test) {
    false => test;
    dac =< accum;
    dac =< dcblock;

    adc => accum;
    adc => dcblock;
    true => sweep.stop;
  }

	int i;
  for (auto s : response) {
    -7.5 / 2 + 7.5 / WINDOW_SIZE * i => float x;
    Math.sqrt((s $ polar).mag * 25) => float y;
    float z;

//    5 *=> y;

    @(x, y, z) => vertices[i];

    i++;
  }

	((now - start) / second * 0.1 + Math.PI / 2 => Math.cos) => float x;
	((now - start) / second * 0.1 + Math.PI / 2 => Math.sin) => float y;

	if (x > 0) {
		if (y > 0) {
			if (phase != 3) {
				3 => phase;
				now => transition;
			}
			(sunrise + (noon - sunrise) * Math.pow((now - transition) / ms / (1000 * 10 * Math.PI / 2), 1)) / 255. => GG.scene().backgroundColor;
		} else {
			if (phase != 2) {
				2 => phase;
				now => transition;
			}
			(night + (sunrise - night) * Math.pow((now - transition) / ms / (1000 * 10 * Math.PI / 2), 5)) / 255. => GG.scene().backgroundColor;
		}
	} else {
		if (y > 0) {
			if (phase != 0) {
				0 => phase;
				now => transition;
			}
			(noon + (sunset - noon) * Math.pow((now - transition) / ms / (1000 * 10 * Math.PI / 2), 5)) / 255. => GG.scene().backgroundColor;
		} else {
			if (phase != 1) {
				1 => phase;
				now => transition;
			}
			(sunset + (night - sunset) * Math.pow((now - transition) / ms / (1000 * 10 * Math.PI / 2), 0.5)) / 255. => GG.scene().backgroundColor;
		}
	}

	GG.nextFrame() => now;
}
